#include <stdio.h>
#include "wrapper.h"
#include "ParCALFuncs.h"

typedef void *(*ParCALDivVortFuncProc_t) (void *, void *);

NhlErrorTypes ParCALProcTempl_W(ParCALDivVortFuncProc_t proc, const char *proc_name)
{
	/*
	* Input array variables
	*/
	void *u, *v;
	int ndims_u;
	ng_size_t dsizes_u[NCL_MAX_DIMENSIONS];
	int ndims_v;
	ng_size_t dsizes_v[NCL_MAX_DIMENSIONS];
	NclScalar missing_u, missing_v, missing_du, missing_dv;
	NclBasicDataTypes type_u, type_v;
	int has_missing_u, has_missing_v, found_missing_u, found_missing_v;
	/*
	* Output array variables
	*/
	void *res_shnd;
	NclMultiDValData res = NULL;
	int ndims_res;
	ng_size_t *dsizes_res;
	NclBasicDataTypes type_res;
	NclScalar missing_res, missing_dres;
	int has_missing_res;

	/*
	* Misc stuff
	*/
	int i, retval;

	/*
	* Retrieve parameters
	*
	* Note any of the pointer parameters can be set to NULL, which
	* implies you don't care about its value.
	*/
	u = (void*)NclGetArgValue(
		   0,
		   3,
		   &ndims_u, 
		   dsizes_u,
		   &missing_u,
		   &has_missing_u,
		   &type_u,
		   DONT_CARE);

	v = (void*)NclGetArgValue(
		   1,
		   3,
		   &ndims_v, 
		   dsizes_v,
		   &missing_v,
		   &has_missing_v,
		   &type_v,
		   DONT_CARE);

	/*
	* Get output array.
	*/
	res = NclGetArgData(
			2,
			3,
			1);
	ndims_res = res->multidval.n_dims;
	dsizes_res = res->multidval.dim_sizes;
	missing_res = res->multidval.missing_value.value;
	has_missing_res = res->multidval.missing_value.has_missing;
	type_res = res->multidval.data_type;

	/*
	* The grids coming in must be at least 2-dimensional and have the same # of
	* dimensions.
	*/
	if( ndims_u != ndims_v || ndims_u < 2 ) {
		NhlPError(NhlFATAL,NhlEUNKNOWN,"%s: The input arrays must be at least 2-dimensional and have the same number of dimensions", proc_name);
		return(NhlFATAL);
	}
	for( i = 0; i < ndims_u; i++ ) {
		if( dsizes_u[i] != dsizes_v[i] ) {
		  NhlPError(NhlFATAL,NhlEUNKNOWN,"%s: The input arrays must have the same dimension sizes", proc_name);
		  return(NhlFATAL);
		}
	}
	/*
	* The output array must also be at least 2-dimensional.
	*/
	if( ndims_res != ndims_u  ) {
		NhlPError(NhlFATAL,NhlEUNKNOWN,"%s: The input/output arrays must have the same number of dimensions", proc_name);
		return(NhlFATAL);
	}
	for( i = 0; i < ndims_u; i++ ) {
		if( dsizes_res[i] != dsizes_u[i] ) {
		  NhlPError(NhlFATAL,NhlEUNKNOWN,"%s: The input/output arrays must have the same dimension sizes", proc_name);
		  return(NhlFATAL);
		}
	}
	if(type_res != NCL_float && type_res != NCL_double && type_res != NCL_pdouble) {
		NhlPError(NhlFATAL,NhlEUNKNOWN,"%s: The output array must be float or double", proc_name);
		return(NhlFATAL);
	}

	/* FIXME: We should also handle non-distributed data */
	res_shnd = (*proc)(u, v);
	res->data.shnd = res_shnd;
	res->multidval.data_type = NCL_pdouble;

	retval = NclSetArgSMode(2, 3, NCLOBJDISTRIB);
	if(retval != NhlNOERROR){
		NhlPError(NhlFATAL, NhlEUNKNOWN, "%s: Unable to set the output array as a distributed array", proc_name);
		return NhlFATAL;
	}

	return(NhlNOERROR);
}


NhlErrorTypes ParCALFuncTempl_W(ParCALDivVortFuncProc_t func, const char *func_name)
{
/*
 * Input array variables
 */
  void *u, *v;
  int ndims_u;
  ng_size_t dsizes_u[NCL_MAX_DIMENSIONS];
  int ndims_v;
  ng_size_t dsizes_v[NCL_MAX_DIMENSIONS];
  NclBasicDataTypes type_u, type_v;
  NclScalar missing_u, missing_v;
  int has_missing_u, has_missing_v, found_missing_u, found_missing_v;
/*
 * Output array variables
 */
  void *res;
  NclBasicDataTypes type_res;
  NclScalar missing_res, missing_dres;

  int i;

/*
 * Retrieve parameters
 *
 * Note any of the pointer parameters can be set to NULL, which
 * implies you don't care about its value.
 */
  u = (void*)NclGetArgValue(
           0,
           2,
           &ndims_u, 
           dsizes_u,
           &missing_u,
           &has_missing_u,
           &type_u,
           DONT_CARE);
  v = (void*)NclGetArgValue(
           1,
           2,
           &ndims_v, 
           dsizes_v,
           &missing_v,
           &has_missing_v,
           &type_v,
           DONT_CARE);

/*
 * The grids coming in must be at least 2-dimensional and have the same # of
 * dimensions.
 */
  if( ndims_u != ndims_v || ndims_u < 2 ) {
    NhlPError(NhlFATAL,NhlEUNKNOWN,"%s: The input arrays must be at least 2-dimensional and have the same number of dimensions", func_name);
    return(NhlFATAL);
  }
  for( i = 0; i < ndims_u; i++ ) {
    if( dsizes_u[i] != dsizes_v[i] ) {
      NhlPError(NhlFATAL,NhlEUNKNOWN,"%s: The input arrays must have the same dimension sizes", func_name);
      return(NhlFATAL);
    }
  }

  /* FIXME: Check the types of u and v */

  type_res = NCL_pdouble;
  /* FIXME: Handle missing values */

  res = (*func)(u, v);

  return(NclReturnValue(res,ndims_u,dsizes_u,NULL,type_res,0));
}

NhlErrorTypes puv2vrf_W( void )
{
	return ParCALProcTempl_W(ParCALUVToVort, "uv2vrf");
}

NhlErrorTypes puv2vrF_W( void )
{
	return ParCALFuncTempl_W(ParCALUVToVort, "uv2vrF");
}

NhlErrorTypes puv2vrg_W( void )
{
	return ParCALProcTempl_W(ParCALUVToVort, "uv2vrg");
}

NhlErrorTypes puv2vrG_W( void )
{
	return ParCALFuncTempl_W(ParCALUVToVort, "uv2vrG");
}

NhlErrorTypes puv2dvf_W( void )
{
	return ParCALProcTempl_W(ParCALUVToDiv, "uv2dvf");
}

NhlErrorTypes puv2dvF_W( void )
{
	return ParCALFuncTempl_W(ParCALUVToDiv, "uv2dvF");
}

NhlErrorTypes puv2dvg_W( void )
{
	return ParCALProcTempl_W(ParCALUVToDiv, "uv2dvg");
}

NhlErrorTypes puv2dvG_W( void )
{
	return ParCALFuncTempl_W(ParCALUVToDiv, "uv2dvG");
}

NhlErrorTypes puv2vra_W( void )
{
	return ParCALProcTempl_W(ParCALUVToVort, "uv2vra");
}

NhlErrorTypes puv2vrA_W( void )
{
	return ParCALFuncTempl_W(ParCALUVToVort, "uv2vrA");
}

NhlErrorTypes puv2dva_W( void )
{
	return ParCALProcTempl_W(ParCALUVToDiv, "uv2dva");
}

NhlErrorTypes puv2dvA_W( void )
{
	return ParCALFuncTempl_W(ParCALUVToDiv, "uv2dvA");
}
